///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _TCPHANDLER_CPP
#define _TCPHANDLER_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN
#define _WIN32_WINNT 0x0500

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WinSock2.H>
#include <WinSock.H>
#include <Stdio.H>
#include <Stdlib.H>

#include "CSockSrvr.H"

//NON-STANDARD DEFINITIONS (BEGIN)

#include "../../SharedSource/Debug.H"
#include "../../SharedClasses/CMemPool/CMemPool.H"

#include "../../SharedSource/Debug.H"
#include "../../SharedSource/NSWFL.H"
#include "../../SharedSource/Common.H"

#include "../Source/Routines.H"

//NON-STANDARD DEFINITIONS (END)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

HANDLE ghEvent_ACK = NULL;
HANDLE ghEvent_TCPHandler = NULL;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSockSrvr::Start_ACKThread(void)
{
    #ifdef _DEBUG_SOCK_SERVER
    printf("DEBUG: [In Proc]> CSockSrvr::Start_ACKThread.\n");
    #endif

    ghEvent_ACK = CreateEvent(NULL, TRUE, FALSE, "ACK_Thread_Function");

    if((hcACK_Thread_Handle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)ACK_Thread_Function, (LPVOID) this, 0, (LPDWORD) &dwcACK_Thread_ID)) == NULL)
    {
        FatalError("CreateThread failed.");
        CloseHandle(ghEvent_ACK);
        return false;
    }

	SetThreadPriority(hcTCPHandler_Thread_Handle, THREAD_PRIORITY_ABOVE_NORMAL);

	if(WaitForSingleObject(ghEvent_ACK, 5000) == WAIT_TIMEOUT)
    {
        CloseHandle(ghEvent_ACK);
        return false;
    }

    CloseHandle(ghEvent_ACK);
    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSockSrvr::Stop_ACKThread(void)
{
    #ifdef _DEBUG_SOCK_SERVER
    printf("DEBUG: [In Proc]> CSockSrvr::Stop_ACKThread.\n");
    #endif

    DWORD dwExitCode = 0;

    if(bcACKActive)
    {
        bcShutdownACK = true;

        while(GetExitCodeThread(hcACK_Thread_Handle, &dwExitCode))
        {
            if(dwExitCode != STILL_ACTIVE)
            {
                CloseHandle(hcACK_Thread_Handle);
                dwcACK_Thread_ID = 0;
                return true;
            }
            Sleep(1);
        }

        FatalError("GetExitCodeThread failed.");

        return false;
    }

    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSockSrvr::Start_TCPHandler(void)
{
    #ifdef _DEBUG_SOCK_SERVER
    printf("DEBUG: [In Proc]> CSockSrvr::Start_TCPHandler.\n");
    #endif

	Start_ACKThread();

	ghEvent_TCPHandler = CreateEvent(NULL, TRUE, FALSE, "TCPHandler_Thread_Function");

    if((hcTCPHandler_Thread_Handle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)TCPHandler_Thread_Function, (LPVOID) this, 0, (LPDWORD) &dwcTCPHandler_Thread_ID)) == NULL)
    {
        FatalError("CreateThread failed.");
        CloseHandle(ghEvent_TCPHandler);
        return false;
    }

	SetThreadPriority(hcTCPHandler_Thread_Handle, THREAD_PRIORITY_ABOVE_NORMAL);

    if(WaitForSingleObject(ghEvent_TCPHandler, 5000) == WAIT_TIMEOUT)
    {
        CloseHandle(ghEvent_TCPHandler);
        return false;
    }

    CloseHandle(ghEvent_TCPHandler);
    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CSockSrvr::Stop_TCPHandler(void)
{
    #ifdef _DEBUG_SOCK_SERVER
    printf("DEBUG: [In Proc]> CSockSrvr::Stop_TCPHandler.\n");
    #endif

    DWORD dwExitCode = 0;

    if(bcTCPHandlerActive)
    {
        bcShutdownTCPHandler = true;

        while(GetExitCodeThread(hcTCPHandler_Thread_Handle, &dwExitCode))
        {
            if(dwExitCode != STILL_ACTIVE)
            {
                CloseHandle(hcTCPHandler_Thread_Handle);
                dwcTCPHandler_Thread_ID = 0;
				Stop_ACKThread();
                return true;
            }
            Sleep(1);
        }

        FatalError("GetExitCodeThread failed.");

		Stop_ACKThread();
        return false;
    }

	Stop_ACKThread();
    return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI ACK_Thread_Function(LPVOID pvThread)
{
    #ifdef _DEBUG_SOCK_SERVER
    printf("DEBUG: [In Proc]> ACK_Thread_Function.\n");
    #endif

    //---------------------(Initialize Thread [Begin])----------
	CSockSrvr* pSockSrvr = (CSockSrvr *) pvThread;

    pSockSrvr->bcShutdownACK = false;
    pSockSrvr->bcACKActive = true;

    SetEvent(ghEvent_ACK);
    //---------------------(Initialize Thread [END])----------

	DWORD dwACKCount = 0;
	int iClient = 0;

	while(!pSockSrvr->bcShutdownACK)
	{
		iClient = 0;

		while(iClient < pSockSrvr->icMaxClients && !pSockSrvr->bcShutdownACK)
		{
			if(pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient])
			{
				if(pSockSrvr->bcSendACK[iClient])
				{
					pSockSrvr->bcSendACK[iClient] = false;

					if(gbDebugMode)
					{
						WriteLogEx(pSockSrvr->icClientID[iClient], "Sent ACK response.", EVENT_NONE);
					}

					if(!pSockSrvr->SetNextSendData(iClient, ACK_PACKET_RESPOND))
					{
						WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to respond to ACK.", EVENT_NONE);
						pSockSrvr->bcDisconnect[iClient] = true;
					}
				}

				if(dwACKCount >= gdwACKInterval && dwACKCount >= 1000 && CCI.bIsAuthenticated
					&& pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient]
				)
				{
					if(pSockSrvr->bcACKCheck[iClient])
					{
						WriteLogEx(pSockSrvr->icClientID[iClient], "Peer failed to respond to ACK request.", EVENT_WARN);
						pSockSrvr->bcDisconnect[iClient] = true;	
					}
					else{
						if(gbDebugMode)
						{
							WriteLogEx(pSockSrvr->icClientID[iClient], "Sent ACK request.", EVENT_NONE);
						}
						if(!pSockSrvr->SetNextSendData(iClient, ACK_PACKET_REQUEST))
						{
							WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to send the ACK packet.", EVENT_ERROR);
							pSockSrvr->bcDisconnect[iClient] = true;	
						}
						pSockSrvr->bcACKCheck[iClient] = true;
					}
				}
			}

			iClient++;
			//Sleep(1);
		}

		if(dwACKCount >= gdwACKInterval)
		{
			dwACKCount = 0;
		}

		dwACKCount++;

		Sleep(1);
	}

	pSockSrvr->bcACKActive = false;
	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD WINAPI TCPHandler_Thread_Function(LPVOID pvThread)
{
    #ifdef _DEBUG_SOCK_SERVER
    printf("DEBUG: [In Proc]> TCPHandler_Thread_Function.\n");
    #endif

    //---------------------(Initialize Thread [Begin])----------
	CSockSrvr* pSockSrvr = (CSockSrvr *) pvThread;

    pSockSrvr->bcShutdownTCPHandler = false;
    pSockSrvr->bcTCPHandlerActive = true;
	pSockSrvr->bcAcceptConnections = true;

	pSockSrvr->TimeOut.tv_sec	= 0;
	pSockSrvr->TimeOut.tv_usec = 250;

	FD_ZERO(&pSockSrvr->ReadFDS);
	FD_ZERO(&pSockSrvr->WriteFDS);
	FD_ZERO(&pSockSrvr->ExceptFDS);

    SetEvent(ghEvent_TCPHandler);
    //---------------------(Initialize Thread [END])----------

	int iClient = 0;

    while(!pSockSrvr->bcShutdownTCPHandler)
    {
        // Zero the Read, Write and Except
    	FD_ZERO(&pSockSrvr->ReadFDS);
    	FD_ZERO(&pSockSrvr->WriteFDS);
    	FD_ZERO(&pSockSrvr->ExceptFDS);

        if(!pSockSrvr->IsListeningServer)
        {
            while(pSockSrvr->icCurrentClients == 0)
            {
                if(pSockSrvr->bcShutdownTCPHandler)
                {
                    break;
                }
                Sleep(TCPHANDLER_MS_WAIT);
            }
            if(pSockSrvr->bcShutdownTCPHandler)
            {
                break;
            }
        }
        else {
            // Set LocalListenSocket socket to Read, Except
    		FD_SET(pSockSrvr->sckcListenSocket, &pSockSrvr->ReadFDS);
    		FD_SET(pSockSrvr->sckcListenSocket, &pSockSrvr->ExceptFDS);
        }

		// Set all connected clients to Read, Write and Except
		if(pSockSrvr->icCurrentClients > 0)
		{
			iClient = 0;
			while(iClient < pSockSrvr->icMaxClients)
			{
				if( pSockSrvr->bcConnected[iClient] == true)
				{
					FD_SET(pSockSrvr->sckcSocket[iClient], &pSockSrvr->ReadFDS);
					FD_SET(pSockSrvr->sckcSocket[iClient], &pSockSrvr->WriteFDS);
					FD_SET(pSockSrvr->sckcSocket[iClient], &pSockSrvr->ExceptFDS);
				}
				iClient++;
			}
		}

		// Socket Select...
		if(select(-1, &pSockSrvr->ReadFDS, &pSockSrvr->WriteFDS, &pSockSrvr->ExceptFDS, &pSockSrvr->TimeOut) == SOCKET_ERROR)
		{
			FatalError("TCPHandler_Thread_Function :: Socket select error.");
			return -1;
		}

        if(pSockSrvr->IsListeningServer)
        {
			if(pSockSrvr->bcAcceptConnections)
			{
    			// Accept any incoming connections.
				if(TryEnterCriticalSection(&pSockSrvr->CriticalSection1))
				{
    				pSockSrvr->DoAccept();
					LeaveCriticalSection(&pSockSrvr->CriticalSection1);
				}
			}
        }

		// Send and receive any pending data.
		if(pSockSrvr->icCurrentClients > 0)
		{
    		iClient = 0;
			while(iClient < pSockSrvr->icMaxClients)
			{
				if(pSockSrvr->bcConnected[iClient] == true)
				{
                	if( FD_ISSET(pSockSrvr->sckcSocket[iClient], &pSockSrvr->WriteFDS) )
                	{
					    if(TryEnterCriticalSection(&pSockSrvr->CriticalSection1))
						{
	    					pSockSrvr->DoSend(iClient);
							LeaveCriticalSection(&pSockSrvr->CriticalSection1);
						}
                    }

                	if( FD_ISSET(pSockSrvr->sckcSocket[iClient], &pSockSrvr->ReadFDS) )
                	{
					    if(TryEnterCriticalSection(&pSockSrvr->CriticalSection1))
						{
	    					pSockSrvr->DoRecv(iClient);
							LeaveCriticalSection(&pSockSrvr->CriticalSection1);
						}
                    }

                	if( FD_ISSET(pSockSrvr->sckcSocket[iClient], &pSockSrvr->ExceptFDS) )
                	{
        				pSockSrvr->bcDisconnect[iClient] = true;
                    }
				}
				iClient++;
			}

    		// Disconnect any pending disconnects.
			if(pSockSrvr->icCurrentClients > 0)
			{
    			iClient = 0;
				while(iClient < pSockSrvr->icMaxClients)
				{
					if( pSockSrvr->bcDisconnect[iClient] == true)
					{
						pSockSrvr->DisconnectClient(iClient);
					}
					iClient++;
				}
			}
		}

        Sleep(TCPHANDLER_MS_WAIT);
	}

    pSockSrvr->bcTCPHandlerActive = false;

	return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
