///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _HANDSHAKE_CPP
#define _HANDSHAKE_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"

#include "../CSockSrvr/CSockSrvr.H"

#include "../../SharedSource/Debug.H"
#include "../../SharedSource/NSWFL.H"
#include "../../SharedSource/Common.H"
#include "../../SharedSource/HashKey.H"
#include "../../SharedSource/CRC32.H"

#include "Entry.H"
#include "Init.H"
#include "Routines.H"
#include "WinService.H"
#include "HandShake.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void BinPrint(const char *sData, int iLen)
{
	int iRPos = 0;
	while(iRPos < iLen)
	{
		printf("%X\n", sData[iRPos]);
		iRPos++;
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	int PerformHandShake(int iClient, char *sRecvBuf, int iRecvBufSz)

	Performs a hand shake. Also know as Client/Server Authentication.

	Possible return Values:
		AUTH_FAILED   //The authentication process failed.
		AUTH_SUCCESS  //The authentication process was a success.
		AUTH_OK       //The authentication process is still in process, all is well.
		AUTH_ERROR    //The authentication process failed due to an error.
*/
int PerformHandShake(CSockSrvr *pSockSrvr, int iClient, char *sCmdBuf, int iCmdBufSz)
{
    char sCmdData[SENDBUFSZ + RECVBUFSZ];
    char sSendBuf[SENDBUFSZ + 1];
    char sTemp[SENDBUFSZ + RECVBUFSZ];

	int iCmdDataSz = 0;
	int iSendBufSz = 0;
	int iCmdFlagLength = 0;

	if((iCmdFlagLength = CmdCmp(sCmdBuf, "::RawHash->")))
    {
		if(gbDebugMode)
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Server requested key hash.", EVENT_NONE);
		}
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		HashKey((const unsigned char *)sCmdData, (unsigned char *)sCmdData, iCmdDataSz, iCmdDataSz);
		pSockSrvr->SetNextSendDataEx(iClient, sCmdData, iCmdDataSz);
		return AUTH_OK;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::UpdateSchedule->")))
    {
		if(gbDebugMode)
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Server is updating the schedule.", EVENT_NONE);
		}

		CLIENT_SCHEDULE CS;

		if((iCmdBufSz - iCmdFlagLength) != sizeof(CS))
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Could not update the schedule, structure size conflict.", EVENT_NONE);
			pSockSrvr->SetNextSendData(iClient, "::ScheduleUpdateFailed");
			return AUTH_ERROR;
		}

		iCmdDataSz = BreakCmdFromDataEx(sCmdBuf, iCmdFlagLength, iCmdBufSz, &CS);

		if(iCmdDataSz != sizeof(CS))
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Could not update the schedule, structure size conflict.", EVENT_NONE);
			pSockSrvr->SetNextSendData(iClient, "::ScheduleUpdateFailed");
			return AUTH_ERROR;
		}

		gbWeekDays[0] = CS.bSunday;
		gbWeekDays[1] = CS.bMonday;
		gbWeekDays[2] = CS.bTuesday;
		gbWeekDays[3] = CS.bWednesday;
		gbWeekDays[4] = CS.bThursday;
		gbWeekDays[5] = CS.bFriday;
		gbWeekDays[6] = CS.bSaturday;

		gdwStartTime = CS.dwBeginTime;
		gdwStopTime = CS.dwEndTime;

		gdwConnectInterval = CS.iIncrement;

		if(SaveRegistryValues())
		{
			pSockSrvr->SetNextSendData(iClient, "::ScheduleUpdateSuccessful");
			return AUTH_OK;
		}
		else{
			pSockSrvr->SetNextSendData(iClient, "::ScheduleUpdateFailed");
		}

		return AUTH_ERROR;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::RequestAuthString")))
    {
		if(gbDebugMode)
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Server requested authentication.", EVENT_NONE);
		}

		int iTempLen = strlen(gsAuthString);

		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		CCI.SC.CipherEx(gsAuthString, sTemp, iTempLen);
		iSendBufSz = AppendDataToCmd("::AuthString->", sTemp, iTempLen, sSendBuf);
		pSockSrvr->SetNextSendDataEx(iClient, sSendBuf, iSendBufSz);
		return AUTH_OK;
	}
    else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::NewAuthKey->")))
    {
		char sNewAuthKey_Raw[MAX_KEY_LENGTH];

		if(gbDebugMode)
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Server issued new cryptography key.", EVENT_NONE);
		}

		int iTempLen = strlen(gsAuthString);

		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

		CCI.SC.CipherEx(sCmdData, sNewAuthKey_Raw, iCmdDataSz);
		sNewAuthKey_Raw[iCmdDataSz] = '\0';

		CCI.SC.DestroyCryptography();

		if(!CCI.SC.InitializeCryptographyEx(sNewAuthKey_Raw, iCmdDataSz, true))
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to Initialize the cryptography set.", EVENT_ERROR);
		}

		CCI.SC.CipherEx(gsAuthString, sTemp, iTempLen);

		iSendBufSz = AppendDataToCmd("::NewKeyAuth->", sTemp, iTempLen, sSendBuf);

		pSockSrvr->SetNextSendDataEx(iClient, sSendBuf, iSendBufSz);
		return AUTH_OK;
	}
    else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::RequestVersion")))
    {
		int iTempLen = strlen(gsFileVersion);

		CCI.SC.CipherEx(gsFileVersion, sTemp, iTempLen);
		iSendBufSz = AppendDataToCmd("::Version->", sTemp, iTempLen, sSendBuf);
		pSockSrvr->SetNextSendDataEx(iClient, sSendBuf, iSendBufSz);
		return AUTH_OK;
	}
    else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::RequestCompanyName")))
    {
		int iTempLen = strlen(gsCompanyName);

		CCI.SC.CipherEx(gsCompanyName, sTemp, iTempLen);
		iSendBufSz = AppendDataToCmd("::CompanyName->", sTemp, iTempLen, sSendBuf);
		pSockSrvr->SetNextSendDataEx(iClient, sSendBuf, iSendBufSz);
		return AUTH_OK;
	}
    else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::RequestPassword")))
    {
		if(gdwACKInterval > 0)
		{
			//Send misc. options.
			pSockSrvr->SetNextSendData(iClient, "::Option-UseACK");
		}

		int iTempLen = strlen(gsCompanyPassword);

		CCI.SC.CipherEx(gsCompanyPassword, sTemp, iTempLen);
		iSendBufSz = AppendDataToCmd("::Password->", sTemp, iTempLen, sSendBuf);
		pSockSrvr->SetNextSendDataEx(iClient, sSendBuf, iSendBufSz);
		return AUTH_OK;
	}
    else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::AuthenticationSuccess")))
    {
		WriteLogEx(pSockSrvr->icClientID[iClient], "Authentication success.", EVENT_NONE);
		return AUTH_SUCCESS;
	}
    else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::AutoUpdate->")))
    {
		WriteLogEx(pSockSrvr->icClientID[iClient], "Starting automatic update process.", EVENT_INFO);

		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

        char sOldExe[MAX_PATH];
        char sNewExe[MAX_PATH];
        char sTempPath[MAX_PATH];

		Get_TempDirectoryEx(sTempPath);

        sprintf_s(sOldExe, sizeof(sOldExe), "%s\\%s", gsPath, "AutoUpdate.Exe");
        sprintf_s(sNewExe, sizeof(sNewExe), "%s\\%s", sTempPath, "SQLEUpdate.Exe");

        CopyFile(sOldExe, sNewExe, FALSE);
        ShellExecute(GetActiveWindow(), "Open", sNewExe, sCmdData, Get_CurrentDirectory(), SW_SHOWNORMAL);
        CommandService(SERVICE_CONTROL_STOP);

		return AUTH_SUCCESS;
	}
    else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::AuthenticationFailed->")))
    {
		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		sCmdData[iCmdDataSz] = '\0';

		sprintf_s(sTemp, sizeof(sTemp), "The server reported an authentication failure: %s", sCmdData);

		WriteLogEx(pSockSrvr->icClientID[iClient], sTemp, EVENT_INFO);
		return AUTH_FAILED;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::AuthenticationFailed")))
    {
		WriteLogEx(pSockSrvr->icClientID[iClient], "The server reported an authentication failure.", EVENT_INFO);
		return AUTH_FAILED;
	}

	return AUTH_ERROR;	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
