///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Copyright  NetworkDLS 2002, All rights reserved
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF 
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A 
// PARTICULAR PURPOSE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _CONSOLEPROCS_CPP
#define _CONSOLEPROCS_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <StdIO.H>
#include <STDLib.H>
#include <PSAPI.H>
#include <SQL.H>

#include "../../SharedSource/Debug.H"
#include "../../SharedSource/NSWFL.H"
#include "../../SharedSource/Common.H"

#include "../../SharedClasses/CMemPool/CMemPool.H"

#include "../Resources/Resource.H"

#include "Entry.H"
#include "Init.H"
#include "WinService.H"
#include "Routines.H"
#include "Console.H"
#include "Console.H"
#include "ConsoleProcs.H"

#include "../../SharedClasses/CMemPool/CMemPool.H"

#include "../CSockSrvr/CSockSrvr.H"

#include "../Dialogs/MainDlg.H"
#include "../Dialogs/AlertDlg.H"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool DisplayConnectionInfo(void)
{
	char sTemp[5120];
	//char sDat[1024];

	WriteCon("Acquiring critical section access.\n");
	EnterCriticalSection(&gServer.CriticalSection1);

	WriteCon("Suspending packet handling process.\n");
	SuspendThread(gServer.hcTCPHandler_Thread_Handle);

	WriteCon(CONSOLEHR"\n\n");

	int iCon = 0;
	int iMaxCon = gServer.GetMaxClients();

	sprintf_s(sTemp, sizeof(sTemp), 
		"Connections: (Current: %d of %d, Total: %d)\n\n",
		gServer.GetCurrentClients(), iMaxCon, gdwTotalConnects);
    WriteCon(sTemp);

	while(iCon < iMaxCon)
	{
		if(gServer.bcConnected[iCon])
		{
			sprintf_s(sTemp, sizeof(sTemp),
				"Connection %d, ID: %d, IP: [%s], Version: [%s]\n"
				"\tAuthenticated: %d, Step: %d\n"
				"\tInitializing: %d, Step: %d\n"
				"\tDB Connect: (Index: %d / Customer: %d)\n"
				"\tBytes: (Sent: %d, Recvd: %d)\n"
				"\n",
				iCon,
				gServer.icClientID[iCon],
				CCI[iCon].sIPAddress,
				CCI[iCon].sClientVersion,
				CCI[iCon].bIsAuthenticated,
				CCI[iCon].dwAuthStep,
				CCI[iCon].bRequestInit,
				CCI[iCon].iInitStep,
				CCI[iCon].bIndexConnected,
				CCI[iCon].bCustomerConnected,
				CCI[iCon].dwBytesSent,
				CCI[iCon].dwBytesRecv
			);
			WriteCon(sTemp);

			WriteCon(CONSOLEHR"\n\n");
		}
		iCon++;
	}

	WriteCon("\n\n"CONSOLEHR"\n");

	WriteCon("Resuming packet handling process.\n");
	ResumeThread(gServer.hcTCPHandler_Thread_Handle);

	WriteCon("Releasing critical section access.\n");
	LeaveCriticalSection(&gServer.CriticalSection1);

	WriteCon("Complete.\n");

	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ConsoleSysStats(void)
{
	char sText[255];
	char sInt[64];

	PROCESS_MEMORY_COUNTERS MyPMC;

	HANDLE hProcess = GetCurrentProcess();

	memset(&MyPMC, 0, sizeof(MyPMC));

    if(!GetProcessMemoryInfo(hProcess, &MyPMC, sizeof(MyPMC)))
	{
		return false;
	}

	FormatInteger(sInt, MyPMC.PageFaultCount);
	sprintf_s(sText, sizeof(sText), "Page Faults: %s\n", sInt);
	WriteCon(sText);

	FormatInteger(sInt, MyPMC.WorkingSetSize / 1024);
	sprintf_s(sText, sizeof(sText), "Memory Used: %sKB\n", sInt);
	WriteCon(sText);

	FormatInteger(sInt, MyPMC.QuotaPagedPoolUsage / 1024);
	sprintf_s(sText, sizeof(sText), "Paged Pool Usage: %sKB\n", sInt);
	WriteCon(sText);

	FormatInteger(sInt, MyPMC.QuotaPeakNonPagedPoolUsage / 1024);
	sprintf_s(sText, sizeof(sText), "Non-Paged Pool Usage: %sKB\n", sInt);
	WriteCon(sText);

	FormatInteger(sInt, MyPMC.PagefileUsage / 1024);
	sprintf_s(sText, sizeof(sText), "Page File Usage: %sKB\n", sInt);
	WriteCon(sText);

	FormatInteger(sInt, MyPMC.PeakWorkingSetSize / 1024);
	sprintf_s(sText, sizeof(sText), "Peak Memory Usage: %sKB\n", sInt);
	WriteCon(sText);

	FormatInteger(sInt, MyPMC.QuotaPeakPagedPoolUsage / 1024);
	sprintf_s(sText, sizeof(sText), "Peak Paged Pool Usage: %sKB\n", sInt);
	WriteCon(sText);

	FormatInteger(sInt, MyPMC.QuotaPeakNonPagedPoolUsage / 1024);
	sprintf_s(sText, sizeof(sText), "Peak Non-Paged Pool Usage: %sKB\n", sInt);
	WriteCon(sText);

	FormatInteger(sInt, MyPMC.PeakPagefileUsage / 1024);
	sprintf_s(sText, sizeof(sText), "Peak Page File Usage: %sKB\n", sInt);
	WriteCon(sText);

	//CloseHandle() IS NOT necessary because GetCurrentProcess()
	//	retrieves a pseudo handle for the current process.
	//CloseHandle(hProcess); 

	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool DumpSQLStatsToConsole(HSTMT hStmt)
{
    return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool DumpSQLResultsToConsole(HSTMT hStmt)
{
    return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool AskForConfirmation(const char *sMsg)
{
	if(sMsg)
	{
		WriteCon(sMsg);
	}
	else{
		WriteCon("Are you sure?\n");
	}

	WriteCon("(Y/N): ");

	char sAwnser[30];

	if(GetConsoleInput(sAwnser, sizeof(sAwnser)))
	{
		return (_strcmpi(sAwnser, "Y") == 0 || _strcmpi(sAwnser, "Yes") == 0);
	}

	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ConsoleConnect(void)
{
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void TestAlertDlg(const char *sText)
{
	StartAlertDlg("Test", sText, true);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef _DEBUG_MEMPOOL
	void PrintMemMap(void)
	{
		int iSlot = 0;
		int iAlloc = 0;

		char sText[1024];
		
		WriteCon("-----------------------------------------------------\n");
		WriteCon("Allocated Memory Map:\n\n");
		sprintf_s(sText, sizeof(sText), "Allocations %d of %d.\n", 
			gMem.GetMemAllocationCount(), gMem.GetSlotAllocationCount());
		WriteCon(sText);

		//Free all the memory stored in the AllocPool structure.
		while(iSlot < gMem.UserAlloc.iAllocated)
		{
			if(gMem.UserAlloc.AllocPool[iSlot].bFree == false)
			{
				sprintf_s(sText, sizeof(sText), "Allocation: %d, Slot: %d, Address: 0x%X\n",
					iAlloc++, iSlot, gMem.UserAlloc.AllocPool[iSlot].iAddress);
				WriteCon(sText);
			}
			iSlot++;
		}

		WriteCon("-----------------------------------------------------\n");
	}
#endif

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

DWORD ProcessConsoleCoreCommand(char *sCommand)
{
	char sTempText[1024];

    if(_strcmpi(sCommand, "help") == 0)
    {
        WriteCon(" Version\t Displays version info.\n");
        WriteCon(" Cls\t\t Clears the screen.\n");
        WriteCon(" Connections\t Displays conections info.\n");
        WriteCon(" KillAll\t Kills all current client connections.\n");
        WriteCon(" Kill [ID]\t Disconnects a specified client connection.\n");
        //WriteCon(" Connect\t Connects to a database and execute commands.\n");
        WriteCon(" Exit\t\t Closes the console.\n");
        WriteCon(" Start\t\t Starts the server.\n");
        WriteCon(" Stop\t\t Stoppes the server.\n");
        WriteCon(" FlushIO\t Flushes all IO streams to disk.\n");
        WriteCon(" ReadRegistry\t Re-Reads all registry values.\n");
        WriteCon(" WriteRegistry\t Writes all registry values.\n");
        WriteCon(" PauseTCP\t Pauses the TCPLoop thread.\n");
        WriteCon(" ResumeTCP\t Resumes the TCPLoop thread.\n");
        WriteCon(" Accept:ON\t Accepts incomming connections.\n");
        WriteCon(" Accept:OFF\t Rejects incomming connections.\n");
		#ifdef _DEBUG_MEMPOOL
			WriteCon(" MemMap\t\t Prints the allocation tables.\n");
		#endif
		return CONSOLE_CORE_RES_OK;
    }
	#ifdef _DEBUG_MEMPOOL
		else if(_strcmpi(sCommand, "MemMap") == 0)
		{
			PrintMemMap();
			return CONSOLE_CORE_RES_OK;
		}
	#endif
	else if(_strcmpi(sCommand, "CLS") == 0)
    {
        system("cls");
        return CONSOLE_CORE_RES_OK;
    }
	else if(_strcmpi(sCommand, "SysStats") == 0)
    {
        if(!ConsoleSysStats())
		{
			WriteCon("ConsoleSysStats failed.\n");
		}
        return CONSOLE_CORE_RES_OK;
    }
	else if(_strcmpi(sCommand, "Accept:ON") == 0)
    {
        gServer.bcAcceptConnections = true;
        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "Accept:Off") == 0)
    {
        gServer.bcAcceptConnections = false;
        return CONSOLE_CORE_RES_OK;
    }
	else if(_strcmpi(sCommand, "PauseTCP") == 0)
    {
		WriteCon("Acquiring critical section access.\n");
		EnterCriticalSection(&gServer.CriticalSection1);

		WriteCon("Suspending packet handling process.\n");
		SuspendThread(gServer.hcTCPHandler_Thread_Handle);

		WriteCon("Releasing critical section access.\n");
		LeaveCriticalSection(&gServer.CriticalSection1);

        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "ResumeTCP") == 0)
    {
		WriteCon("Acquiring critical section access.\n");
		EnterCriticalSection(&gServer.CriticalSection1);

		WriteCon("Resuming packet handling process.\n");
		ResumeThread(gServer.hcTCPHandler_Thread_Handle);

		WriteCon("Releasing critical section access.\n");
		LeaveCriticalSection(&gServer.CriticalSection1);

        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "ReadRegistry") == 0)
    {
		WriteCon("Acquiring critical section access.\n");
		EnterCriticalSection(&gServer.CriticalSection1);

		WriteCon("Suspending packet handling process.\n");
		SuspendThread(gServer.hcTCPHandler_Thread_Handle);

		if(GetRegistryValues())
		{
            WriteCon("Registry vlaues re-read.\n");
		}
		else{
			WriteCon("Error reading registry values.\n");
		}

		WriteCon("Resuming packet handling process.\n");
		ResumeThread(gServer.hcTCPHandler_Thread_Handle);

		WriteCon("Releasing critical section access.\n");
		LeaveCriticalSection(&gServer.CriticalSection1);

        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "WriteRegistry") == 0)
    {
		WriteCon("Acquiring critical section access.\n");
		EnterCriticalSection(&gServer.CriticalSection1);

		WriteCon("Suspending packet handling process.\n");
		SuspendThread(gServer.hcTCPHandler_Thread_Handle);

        if(SaveRegistryValues())
		{
			WriteCon("Registry vlaues written.\n");
		}
		else{
			WriteCon("Error writing registry values.\n");
		}

		WriteCon("Resuming packet handling process.\n");
		ResumeThread(gServer.hcTCPHandler_Thread_Handle);

		WriteCon("Releasing critical section access.\n");
		LeaveCriticalSection(&gServer.CriticalSection1);

        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "FlushIO") == 0)
    {
        sprintf_s(sTempText, sizeof(sTempText), "Flushed %d IO streames to disk.\n", _flushall());
		WriteCon(sTempText);
        return CONSOLE_CORE_RES_OK;
    }
	else if(_strnicmp(sCommand, "Kill:", 5) == 0)
    {
		DWORD dwClientID = -1;
		
		if((dwClientID = atol(sCommand + 5)) <= 0)
		{
			sprintf_s(sTempText, sizeof(sTempText), "%d is an invalid connection ID.\n", dwClientID);
			WriteCon(sTempText);			
			return CONSOLE_CORE_RES_OK;
		}

		sprintf_s(sTempText, sizeof(sTempText), "Disconnect the client with ID %d?\n", dwClientID);

		if(AskForConfirmation(sTempText))
		{
			DWORD dwCount = 0;

			while(dwCount < gdwMaxClients)
			{
				if(gServer.icClientID[dwCount] == dwClientID)
				{
					gServer.bcDisconnect[dwCount] = true;
					sprintf_s(sTempText, sizeof(sTempText), "Disconnected connection %d with ID %d.\n", dwCount, dwClientID);
					WriteCon(sTempText);
				}
				dwCount++;
			}
		}
        return CONSOLE_CORE_RES_OK;
    }
	else if(_strcmpi(sCommand, "KillAll") == 0)
    {
		if(AskForConfirmation("Disconnect all existing connections?\n"))
		{
			DWORD dwDisconnects = 0;
			DWORD dwCount = 0;

			while(dwCount < gdwMaxClients)
			{
				if(gServer.bcConnected[dwCount] && !gServer.bcDisconnect[dwCount])
				{
					gServer.bcDisconnect[dwCount] = true;
					dwDisconnects++;
				}
				dwCount++;
			}

			sprintf_s(sTempText, sizeof(sTempText), "Disconnected %d clients.\n", dwDisconnects);
			WriteCon(sTempText);
		}
        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "Start") == 0)
    {
        if(StartServer())
		{
            WriteCon("Server started.\n");
		}
		else{
			WriteCon("Failed to start the server.\n");
		}
        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "Stop") == 0)
    {
        if(StopServer())
		{
            WriteCon("Server stopped.\n");
		}
		else{
			WriteCon("Failed to stop the server.\n");
		}
        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "Version") == 0)
    {
        sprintf_s( sTempText, sizeof(sTempText),
			"%s - [Version: %s] (Built: %s at %s)\n",
			gsTitleCaption, gsFileVersion, __DATE__, __TIME__);
        WriteCon(sTempText);
        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "connections") == 0)
    {
		DisplayConnectionInfo();
        return CONSOLE_CORE_RES_OK;
    }
	else if(_strnicmp(sCommand, "Alert ", 6) == 0)
	{
		TestAlertDlg(sCommand + 6);
	}
	else if(_strcmpi(sCommand, "Connect") == 0)
    {
        ConsoleConnect();
        return CONSOLE_CORE_RES_OK;
    }
    else if(_strcmpi(sCommand, "exit") == 0)
    {
        return CONSOLE_CORE_RES_CLOSE;
    }
    else{
        sprintf_s(sTempText, sizeof(sTempText), "Unknown command: '%s'.\n", sCommand);
        WriteCon(sTempText);
        return CONSOLE_CORE_RES_UNKNOWN;
    }

    return CONSOLE_CORE_RES_ERROR;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
